@@ -1,5 +1,5 @@
 #nhanesA - retrieve data from the CDC NHANES repository
-# Christopher J. Endres 05/02/2022
+# Christopher J. Endres 07/22/2022
 #
 nhanesURL <- 'https://wwwn.cdc.gov/Nchs/Nhanes/'
 dataURL <- 'https://wwwn.cdc.gov/Nchs/Nhanes/search/DataPage.aspx'
@@ -128,6 +128,38 @@ anomalytables2005 <- c('CHLMD_DR', 'SSUECD_R', 'HSV_DR')
 # Internal function to determine if a number is even
 .is.even <- function(x) {x %% 2 == 0}
 
+# Internal function to test for successful html read
+# FUNCTION checkHtml
+# If read_html is successful, then the html is returned.
+# Otherwise return NULL for proper error handling
+.checkHtml <- function(url) {
+  out <- tryCatch(
+    {
+      # when "try" is successful, 'tryCatch()' returns the html 
+      hurl <- xml2::read_html(url)
+      hurl
+    },
+    error=function(cond) {
+      # If there is an error, determine if it's a timeout error or URL error 
+      ccond <- as.character(cond)
+      if(length(grep('imeout',ccond)) > 0) {
+        message("Timeout was reached: No data pulled", "\n")
+      } else if(length(grep('Could not resolve', ccond))>0) {
+        message(cond)
+        #message("Could not resolve host", "\n")
+      } else { message(paste(c("URL ", url, " does not seem to exist"), collapse='')) }   
+      
+      # Return NULL
+      return(NULL)
+    },
+    warning=function(cond) {
+      message(cond)
+      return(NULL)
+    }
+  )
+  return(out)
+}
+
 #xpath <- '//*[@id="ContentPlaceHolder1_GridView1"]'
 xpath <- '//*[@id="GridView1"]'
 
@@ -142,15 +174,17 @@ xpath <- '//*[@id="GridView1"]'
 #' @importFrom magrittr %>%
 #' @param data_group The type of survey (DEMOGRAPHICS, DIETARY, EXAMINATION, LABORATORY, QUESTIONNAIRE).
 #' Abbreviated terms may also be used: (DEMO, DIET, EXAM, LAB, Q).
-#' @param year The year in yyyy format where 1999 <= yyyy <= 2014.
+#' @param year The year in yyyy format where 1999 <= yyyy.
 #' @param nchar Truncates the table description to a max length of nchar.
 #' @param details If TRUE then a more detailed description of the tables is returned (default=FALSE).
 #' @param namesonly If TRUE then only the table names are returned (default=FALSE).
 #' @param includerdc If TRUE then RDC only tables are included in list (default=FALSE).
-#' @return The names of the tables in the specified survey group.
-#' @details Data are retrieved via web scraping using html wrappers from package rvest.
-#' It is often useful to display the table names in an NHANES survey. In effect this
-#' is a convenient way to browse the available NHANES tables.
+#' @return Returns a data frame that contains table attributes. If namesonly=TRUE,
+#' then a character vector of table names is returned.
+#' @details Function nhanesTables retrieves a list of tables and a 
+#' description of their contents from the NHANES website. This provides
+#' a convenient way to browse the available tables. NULL is returned when an
+#' HTML read error is encountered.
 #' @examples
 #' nhanesTables('EXAM', 2007)
 #' \donttest{nhanesTables('LAB', 2009, details=TRUE, includerdc=TRUE)}
@@ -181,7 +215,13 @@ nhanesTables <- function(data_group, year, nchar=100, details = FALSE, namesonly
                 '&CycleBeginYear=', unlist(str_split(as.character(nh_year), '-'))[[1]] , sep='')
   }
   # At this point df contains every table
-  df <- as.data.frame(turl %>% read_html() %>% html_elements(xpath=xpath) %>% html_table())
+  hurl <- .checkHtml(turl)
+  if(is.null(hurl)) {
+    message("Error occurred during read. No tables returned")
+    return(NULL)
+  }
+  df <- as.data.frame(hurl %>% html_elements(xpath=xpath) %>% html_table())
+#  df <- as.data.frame(turl %>% read_html() %>% html_elements(xpath=xpath) %>% html_table())
   # By default we exclude RDC Only tables as those cannot be downloaded
   
   if(year %in% c('P', 'p', 'Y', 'y')) {
@@ -255,10 +295,11 @@ nhanesTables <- function(data_group, year, nchar=100, details = FALSE, namesonly
 #' @param nchar The number of characters in the Variable Description to print. Values are limited to 0<=nchar<=128.
 #' This is used to enhance readability, cause variable descriptions can be very long.
 #' @param namesonly If TRUE then only the variable names are returned (default=FALSE).
-#' @return The names of the tables in the specified survey group
-#' @details Data are retrieved via web scraping using html wrappers from package rvest.
-#' Each data table contains multiple, sometimes more than 100, fields. It is helpful to list the field
-#' descriptions to ascertain quickly if a data table is of interest.
+#' @return Returns a data frame that describes variable attributes for the specified table. If namesonly=TRUE,
+#' then a character vector of the variable names is returned.
+#' @details NHANES tables may contain more than 100 variables. Function nhanesTableVars provides a concise display
+#' of variables for a specified table, which helps to ascertain quickly if the table is of interest. 
+#' NULL is returned when an HTML read error is encountered.
 #' @examples
 #' \donttest{nhanesTableVars('LAB', 'CBC_E')}
 #' \donttest{nhanesTableVars('EXAM', 'OHX_E', details=TRUE, nchar=50)}
@@ -275,7 +316,13 @@ nhanesTableVars <- function(data_group, nh_table, details = FALSE, nchar=100, na
   turl <- str_c(nhanesURL, 'search/variablelist.aspx?Component=', 
                 str_to_title(as.character(nhanes_group[data_group])), 
                 '&CycleBeginYear=', unlist(str_split(as.character(nh_year), '-'))[[1]] , sep='')
-  df <- as.data.frame(turl %>% read_html() %>% html_elements(xpath=xpath) %>% html_table())
+  hurl <- .checkHtml(turl) 
+  if(is.null(hurl)) {
+    message("Error occurred during read. No table variables returned")
+    return(NULL)
+  }
+  df <- as.data.frame(hurl %>% html_elements(xpath=xpath) %>% html_table())
+#  df <- as.data.frame(turl %>% read_html() %>% html_elements(xpath=xpath) %>% html_table())
   
   if(!(nh_table %in% df$Data.File.Name)) {
     stop('Table ', nh_table, ' not present in the ', data_group, ' survey' )
@@ -308,8 +355,10 @@ nhanesTableVars <- function(data_group, nh_table, details = FALSE, nchar=100, na
 #' @importFrom stringr str_c
 #' @param nh_table The name of the specific table to retrieve.
 #' @return The table is returned as a data frame.
-#' @details Downloads a table from the NHANES website in its entirety. NHANES tables 
-#' are stored in SAS '.XPT' format. Function nhanes cannot be used to import limited 
+#' @details Downloads a table from the NHANES website as is, i.e. in its entirety
+#' with no modification or cleansing. NHANES tables 
+#' are stored in SAS '.XPT' format but are imported as a data frame.
+#' Function nhanes cannot be used to import limited 
 #' access data.
 #' @examples 
 #' \donttest{nhanes('BPX_E')}
@@ -474,7 +523,8 @@ nhanesAttr <- function(nh_table) {
 #' @param includerdc If TRUE then RDC only tables are included in list (default=FALSE).
 #' @param nchar Truncates the variable description to a max length of nchar.
 #' @param namesonly If TRUE then only the table names are returned (default=FALSE).
-#' @return A list of tables that match the search terms. 
+#' @return Returns a data frame that describes variables that matched the search terms. If namesonly=TRUE,
+#' then a character vector of table names that contain matched variables is returned. 
 #' @details nhanesSearch is useful to obtain a comprehensive list of relevant tables.
 #' Search terms will be matched against the variable descriptions in the NHANES Comprehensive
 #' Variable Lists.
@@ -498,8 +548,10 @@ nhanesSearch <- function(search_terms=NULL, exclude_terms=NULL, data_group=NULL,
 
   df_initialized = FALSE
   for(i in 1:length(varURLs)) {  
-    vlhtml <- read_html(varURLs[i])
+#    vlhtml <- read_html(varURLs[i])
+    vlhtml <- .checkHtml(varURLs[i])
     
+    if(!is.null(vlhtml)) {
     xpathh <- '//*[@id="GridView1"]/thead/tr'
     hnodes <- html_elements(vlhtml, xpath=xpathh)
     vmcols <- sapply(xml_children(hnodes),xml_text)
@@ -516,10 +568,11 @@ nhanesSearch <- function(search_terms=NULL, exclude_terms=NULL, data_group=NULL,
         df <- rbind(df, dfadd)
       }
     }
+    }
   }
   
-  if(is.null(df)) {
-    stop("No data was found. Perhaps the NHANES url has changed")
+  if(!df_initialized) { # There was no successful match
+    message("Empty result set")
     return(NULL)
   }
   names(df) <- vmcols
@@ -614,7 +667,9 @@ nhanesSearch <- function(search_terms=NULL, exclude_terms=NULL, data_group=NULL,
 #' @param nchar Truncates the variable description to a max length of nchar.
 #' @param details If TRUE then complete table information from the comprehensive
 #' data list is returned (default=FALSE).
-#' @return A list of table names that match the pattern.
+#' @return Returns a character vector of table names that match the given pattern. If details=TRUE,
+#' then a data frame of table attributes is returned. NULL is returned when an
+#' HTML read error is encountered.
 #' @details Searches the Doc File field in the NHANES Comprehensive Data List 
 #' (see https://wwwn.cdc.gov/nchs/nhanes/search/DataPage.aspx) for tables
 #' that match a given name pattern. Only a single pattern may be entered.
@@ -630,8 +685,15 @@ nhanesSearchTableNames <- function(pattern=NULL, ystart=NULL, ystop=NULL, includ
     warning("Multiple patterns entered. Only the first will be matched.")
   }
   
-  df <- data.frame(read_html(dataURL) %>% html_elements(xpath=xpath) %>% html_table())
-  df <- df[grep(paste(pattern,collapse="|"), df$Doc.File),]
+  hurl <- .checkHtml(dataURL)
+  if(is.null(hurl)) {
+    message("Error occurred during read. No table names returned")
+    return(NULL)
+  }
+  df <- data.frame(hurl %>% html_elements(xpath=xpath) %>% html_table())
+#  df <- data.frame(read_html(dataURL) %>% html_elements(xpath=xpath) %>% html_table())
+
+    df <- df[grep(paste(pattern,collapse="|"), df$Doc.File),]
   if(nrow(df)==0) {return(NULL)}
   if(!includerdc) {
     df <- df[!(df$Data.File=='RDC Only'),]
@@ -691,6 +753,8 @@ nhanesSearchTableNames <- function(pattern=NULL, ystart=NULL, ystop=NULL, includ
 #' @param includerdc If TRUE then RDC only tables are included in list (default=FALSE).
 #' @param nchar Truncates the variable description to a max length of nchar.
 #' @param namesonly If TRUE then only the table names are returned (default=TRUE).
+#' @return By default, a character vector of table names that include the specified variable 
+#' is returned. If namesonly=FALSE, then a data frame of table attributes is returned. 
 #' @details The NHANES Comprehensive Variable List is scanned to find all data tables that
 #' contain the given variable name. Only a single variable name may be entered, and only
 #' exact matches will be found.
@@ -711,7 +775,13 @@ nhanesSearchVarName <- function(varname=NULL, ystart=NULL, ystop=NULL, includerd
   df_initialized = FALSE
   
   for(i in 1:length(varURLs)) {
-    tabletree <- varURLs[i] %>% read_html() %>% html_elements(xpath=xpt)
+    
+    hurl <- .checkHtml(varURLs[i])
+    
+    if(!is.null(hurl)) {
+    tabletree <- hurl %>% html_elements(xpath=xpt)    
+#    tabletree <- varURLs[i] %>% read_html() %>% html_elements(xpath=xpt)
+    
     ttlist <- lapply(lapply(tabletree, xml_children), xml_text)
     # Convert the list to a data frame
     
@@ -726,9 +796,11 @@ nhanesSearchVarName <- function(varname=NULL, ystart=NULL, ystop=NULL, includerd
         }
       }
     }
+    }
   }
   
   if(!df_initialized) { # There was no successful match
+    message("Empty result set")
     return(NULL)
   }
   
@@ -806,10 +878,10 @@ nhanesSearchVarName <- function(varname=NULL, ystart=NULL, ystop=NULL, includerd
 #' @param details If TRUE then all available table translation information is displayed (default=FALSE).
 #' @param dxa If TRUE then the 2005-2006 DXA translation table will be used (default=FALSE).
 #' @return The code translation table (or translated data frame when data is defined).
-#' @details Code translation tables are retrieved via webscraping using rvest. 
-#' Many of the NHANES data tables have encoded values. E.g. 1 = 'Male', 2 = 'Female'.
+#' @details Most NHANES data tables have encoded values. E.g. 1 = 'Male', 2 = 'Female'.
 #' Thus it is often helpful to view the code translations and perhaps insert the translated values
-#' in a data frame.
+#' in a data frame. Only a single table may be specified, but multiple variables within that table
+#' can be selected. Code translations are retrieved for each variable. 
 #' @examples
 #' nhanesTranslate('DEMO_B', c('DMDBORN','DMDCITZN'))
 #' nhanesTranslate('BPX_F', 'BPACSZ', details=TRUE)
@@ -825,10 +897,16 @@ nhanesTranslate <- function(nh_table, colnames=NULL, data = NULL, nchar = 32,
   
   get_translation_table <- function(colname, url, details) {
     xpt <- str_c('//*[h3[a[@name="', colname, '"]]]', sep='')
-    tabletree <- url %>% read_html() %>% html_elements(xpath=xpt)
+    
+    hurl <- .checkHtml(url)
+    tabletree <- hurl %>% html_elements(xpath=xpt)
+#    tabletree <- url %>% read_html() %>% html_elements(xpath=xpt)
     if(length(tabletree)==0) { # If not found then try 'id' instead of 'name'
       xpt <- str_c('//*[h3[@id="', colname, '"]]', sep='')
-      tabletree <- url %>% read_html() %>% html_elements(xpath=xpt)
+      
+      hurl <- .checkHtml(url)
+      tabletree <- hurl %>% html_elements(xpath=xpt)
+#      tabletree <- url %>% read_html() %>% html_elements(xpath=xpt)
     }
     if(length(tabletree)>0) {
       tabletrans <- as.data.frame(html_elements(tabletree, 'table') %>% html_table())
@@ -838,10 +916,16 @@ nhanesTranslate <- function(nh_table, colnames=NULL, data = NULL, nchar = 32,
         lcnm <- colname
         stringr::str_sub(lcnm, start=nc, end=nc) <- tolower(stringr::str_sub(lcnm, start=nc, end=nc))
         xpt <- str_c('//*[h3[a[@name="', lcnm, '"]]]', sep='')
-        tabletree <- url %>% read_html() %>% html_elements(xpath=xpt)
+        
+        hurl <- .checkHtml(url)
+        tabletree <- hurl %>% html_elements(xpath=xpt)
+#        tabletree <- url %>% read_html() %>% html_elements(xpath=xpt)
         if(length(tabletree)==0) { # If not found then try 'id' instead of 'name'
           xpt <- str_c('//*[h3[@id="', lcnm, '"]]', sep='')
-          tabletree <- url %>% read_html() %>% html_elements(xpath=xpt)
+          
+          hurl <- .checkHtml(url)
+          tabletree <- hurl %>% html_elements(xpath=xpt)
+#          tabletree <- url %>% read_html() %>% html_elements(xpath=xpt)
         }
         
         if(length(tabletree)>0) {
@@ -862,7 +946,7 @@ nhanesTranslate <- function(nh_table, colnames=NULL, data = NULL, nchar = 32,
       }
       return(tabletrans)
     } else { 
-      warning(c('No translation table is available for ', colname), collapse='')
+      message(paste(c('No translation table is available for ', colname), collapse=''))
       return(NULL)
     }
   }
@@ -877,7 +961,7 @@ nhanesTranslate <- function(nh_table, colnames=NULL, data = NULL, nchar = 32,
     if(nh_year == "Nnyfs"){
       code_translation_url <- str_c("https://wwwn.cdc.gov/Nchs/", nh_year, '/', nh_table, '.htm', sep='')
     } else {
-    code_translation_url <- str_c(nhanesURL, nh_year, '/', nh_table, '.htm', sep='')
+      code_translation_url <- str_c(nhanesURL, nh_year, '/', nh_table, '.htm', sep='')
     }
   }
   translations <- lapply(colnames, get_translation_table, code_translation_url, details)
@@ -933,10 +1017,11 @@ nhanesTranslate <- function(nh_table, colnames=NULL, data = NULL, nchar = 32,
 #' 
 #' @importFrom stringr str_c str_to_title str_split str_sub str_extract_all
 #' @importFrom utils browseURL
-#' @param year The year in yyyy format where 1999 <= yyyy <= 2016.
+#' @param year The year in yyyy format where 1999 <= yyyy.
 #' @param data_group The type of survey (DEMOGRAPHICS, DIETARY, EXAMINATION, LABORATORY, QUESTIONNAIRE).
 #' Abbreviated terms may also be used: (DEMO, DIET, EXAM, LAB, Q).
 #' @param nh_table The name of an NHANES table.
+#' @return No return value
 #' @details browseNHANES will open a web browser to the specified NHANES site.
 #' @examples
 #' browseNHANES()                     # Defaults to the main data sets page
